package main

import (
	"git.joco.dk/sng/fermentord/internal/controllers"
	"git.joco.dk/sng/fermentord/pkg/temperature"
	"github.com/getsentry/sentry-go"
	"github.com/rs/zerolog/log"
	"github.com/spf13/viper"
)

type configuration struct {
	controllers.ControllerConfig

	HTTP struct {
		Port int16 `mapstructure:"port"`
	} `mapstructure:"http"`

	DB struct {
		Local struct {
			DSN string `mapstructure:"dsn"`
		} `mapstructure:"local"`
	} `mapstructure:"db"`
}

func loadConfiguration() *configuration {
	config := &configuration{}

	viper.SetDefault("pid.kp", 2.0)
	viper.SetDefault("pid.ki", 0.0001)
	viper.SetDefault("pid.kd", 2.0)
	viper.SetDefault("db.local.dsn", "/var/lib/fermentord/local.db")
	viper.SetDefault("http.port", 8000)
	viper.SetDefault("db.local.dsn", "./fermentord.db")

	viper.AllowEmptyEnv(true)
	viper.AutomaticEnv()
	viper.AddConfigPath("/etc")
	viper.SetConfigName("fermentord")
	viper.SetConfigType("hcl")

	if err := viper.ReadInConfig(); err != nil {
		log.Debug().
			Err(err).
			Msg("Error loading configuration")
	}

	if err := viper.Unmarshal(&config); err != nil {
		sentry.CaptureException(err)
		log.Fatal().
			Err(err).
			Msg("Error unmarshalling configuration")
	}

	return config
}

func reloadConfiguration(config *configuration, ctrl *controllers.ChamberController) {
	temperature.ConfigUpdates <- []string{
		config.Sensor.Ambient,
		config.Sensor.Chamber,
		config.Sensor.Wort,
	}

	ctrl.ConfigUpdates <- config.ControllerConfig
}
